<?php

/**
 * Auteur : Julian Perez Ramirez
 * Version : 1.0
 * Licence : MIT
 */

namespace App\Controller;

use App\Entity\Item;
use App\Repository\ItemRepository;
use App\Repository\WishListRepository;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Attribute\Route;

use Doctrine\ORM\EntityManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Validator\Validator\ValidatorInterface;

#[Route('/items')]
final class ItemController extends AbstractController
{
    private EntityManagerInterface $entityManager;
    private ItemRepository $itemRepository;
    private WishListRepository $wishListRepository;
    private ValidatorInterface $validator;
    
    public function __construct(
        EntityManagerInterface $entityManager,
        ItemRepository $itemRepository,
        WishListRepository $wishListRepository,
        ValidatorInterface $validator
    ) {
        $this->entityManager = $entityManager;
        $this->itemRepository = $itemRepository;
        $this->wishListRepository = $wishListRepository;
        $this->validator = $validator;
    }

    #[Route('', methods: ['GET'])]
    public function getAllItems(Request $req): JsonResponse
    {
        $wishList = $req->query->get('wishList');
        $wishList = $wishList !== null ? (int) $wishList : null; // Convert to int if provided

        // Validate "onlyBought" as boolean (default: false)
        $onlyBought = filter_var($req->query->get('onlyBought', false), 
            FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE) ?? false;

        // Validate "sortBy" to allow only 'createdAt' or 'price'
        $allowedSortBy = ['createdAt', 'price'];
        $sortBy = $req->query->get('sortBy', 'createdAt');
        if (!in_array($sortBy, $allowedSortBy, true)) {
            $sortBy = 'price';
        }

        // Validate "sort" to allow only 'asc' or 'desc'
        $allowedSort = ['asc', 'desc'];
        $sort = strtolower($req->query->get('sort', 'asc'));
        if (!in_array($sort, $allowedSort, true)) {
            $sort = 'desc';
        }

        // Validate "max" as a positive integer (default: 10)
        $max = filter_var($req->query->get('max', 10), FILTER_VALIDATE_INT,
            ["options" => ["min_range" => 1]]) ?: 10;

        // Validate "page" as a positive integer (default: 1)
        $page = filter_var($req->query->get('page', 1), FILTER_VALIDATE_INT,
            ["options" => ["min_range" => 1]]) ?: 1;

        // Fetch items from repository with filters
        $items = $this->itemRepository->findAllItemsFiltered($wishList, $onlyBought, $sortBy,
             $sort, $max, $page);

        if (!$items) {
            return new JsonResponse(['error' => 'No items found'], Response::HTTP_NOT_FOUND);
        }

        $itemArray = array_map(function ($item) {
            $wishList = $item->getWishList();
            $user = $wishList->getUser(); // Assuming each wishlist has an associated user
        
            return [
                'id' => $item->getId(),
                'wishList' => [
                    'id' => $wishList->getId(),
                    'user' => [
                        'id' => $user->getId(),
                        'username' => $user->getUserName(),
                        'mail' => $user->getEmail(),
                        'role' => $user->getRoles(),
                        'isBlocked' => $user->isBlocked(),
                        'createdAt' => $user->getCreatedAt()->format('Y-m-d H:i:s'),
                    ],
                    'name' => $wishList->getName(),
                    'description' => $wishList->getDescription(),
                    'expirationDate' => $wishList->getExpirationDate()->format('Y-m-d'),
                    'isActive' => $wishList->isActive(),
                    'createdAt' => $wishList->getCreatedAt()->format('Y-m-d H:i:s'),
                ],
                'title' => $item->getTitle(),
                'description' => $item->getDescription(),
                'price' => $item->getPrice(),
                'purchaseUrl' => $item->getPurchaseUrl(),
                'createdAt' => $item->getCreatedAt()->format('Y-m-d H:i:s'),
            ];
        }, $items);
        
        return $this->json([
            'items' => $itemArray,
            'page' => $page,
            'max' => $max,
            'sortBy' => $sortBy,
            'sort' => $sort,
            'onlyBought' => $onlyBought,
            'path' => 'src/Controller/ItemController.php',
        ]);

    }

    #[Route('/{itemId}', methods: ['GET'])]
    public function getItemsById($itemId): JsonResponse
    {
        $item = $this->itemRepository->findOneById($itemId);

        if (!$item) {
            return new JsonResponse(['error' => 'Item not found'], Response::HTTP_NOT_FOUND);
        }

        $wishList = $item->getWishList();
        $user = $wishList->getUser(); // Assuming a wishlist has an associated user

        $itemDTO = [
            'id' => $item->getId(),
            'wishList' => [
                'id' => $wishList->getId(),
                'user' => [
                    'id' => $user->getId(),
                    'username' => $user->getUserName(),
                    'mail' => $user->getEmail(),
                    'role' => $user->getRoles(),
                    'isBlocked' => $user->isBlocked(),
                    'createdAt' => $user->getCreatedAt()->format('Y-m-d H:i:s'),
                ],
                'name' => $wishList->getName(),
                'description' => $wishList->getDescription(),
                'expirationDate' => $wishList->getExpirationDate()->format('Y-m-d'),
                'isActive' => $wishList->isActive(),
                'createdAt' => $wishList->getCreatedAt()->format('Y-m-d H:i:s'),
            ],
            'title' => $item->getTitle(),
            'description' => $item->getDescription(),
            'price' => $item->getPrice(),
            'purchaseUrl' => $item->getPurchaseUrl(),
            'createdAt' => $item->getCreatedAt()->format('Y-m-d H:i:s'),
        ];

        return $this->json([
            'item' => $itemDTO,
            'path' => 'src/Controller/ItemController.php',
        ]);
    }

    // NEW METHODS
    #[Route('', methods: ['POST'])]
    public function createItem(Request $request): JsonResponse
    {
        $data = json_decode($request->getContent(), true);
        
        if (json_last_error() !== JSON_ERROR_NONE) {
            return new JsonResponse(
                ['error' => 'Invalid JSON payload'],
                Response::HTTP_BAD_REQUEST
            );
        }

        // Check required fields
        if (!isset($data['title']) || !isset($data['price']) || !isset($data['wishList'])) {
            return new JsonResponse(
                ['error' => 'Missing required fields: title, price and wishList are required'],
                Response::HTTP_BAD_REQUEST
            );
        }

        $item = new Item();
        
        try {
            $wishList = $this->wishListRepository->findOneById($data['wishList']);
            if (!$wishList) {
                return new JsonResponse(
                    ['error' => 'WishList not found'],
                    Response::HTTP_BAD_REQUEST
                );
            }
            $item->setWishList($wishList);
        } catch (\Exception $e) {
            return new JsonResponse(
                ['error' => 'Invalid wishList: ' . $e->getMessage()],
                Response::HTTP_BAD_REQUEST
            );
        }

        $item->setTitle($data['title']);
        $item->setDescription($data['description'] ?? null);
        $item->setPrice((float)$data['price']);
        $item->setPurchaseUrl($data['purchaseUrl'] ?? null);
        $item->setCreatedAt(new \DateTimeImmutable());

        // Validate using ValidatorInterface
        $errors = $this->validator->validate($item);
        
        if (count($errors) > 0) {
            $errorMessages = [];
            foreach ($errors as $error) {
                $errorMessages[$error->getPropertyPath()] = $error->getMessage();
            }
            return new JsonResponse(
                ['validation_errors' => $errorMessages],
                Response::HTTP_UNPROCESSABLE_ENTITY
            );
        }

        try {
            $this->entityManager->persist($item);
            $this->entityManager->flush();

            return new JsonResponse([
                'id' => $item->getId(),
                'title' => $item->getTitle(),
                'price' => $item->getPrice(),
                'wishListId' => $item->getWishList()->getId(),
                'createdAt' => $item->getCreatedAt()->format('Y-m-d H:i:s'),
                'path' => 'src/Controller/ItemController.php'
            ], Response::HTTP_CREATED);
        } catch (\Exception $e) {
            return new JsonResponse(
                ['error' => 'Failed to create item: ' . $e->getMessage()],
                Response::HTTP_INTERNAL_SERVER_ERROR
            );
        }
    }

    #[Route('/{id}', methods: ['PATCH'])]
    public function updateItem(int $id, Request $request): JsonResponse
    {
        $item = $this->itemRepository->findOneById($id);
        
        if (!$item) {
            return new JsonResponse(
                ['error' => 'Item not found'],
                Response::HTTP_NOT_FOUND
            );
        }

        $data = json_decode($request->getContent(), true);
        
        if (json_last_error() !== JSON_ERROR_NONE) {
            return new JsonResponse(
                ['error' => 'Invalid JSON payload'],
                Response::HTTP_BAD_REQUEST
            );
        }

        if (isset($data['title'])) {
            $item->setTitle($data['title']);
        }
        
        if (isset($data['description'])) {
            $item->setDescription($data['description']);
        }
        
        if (isset($data['price'])) {
            $item->setPrice((float)$data['price']);
        }
        
        if (isset($data['purchaseUrl'])) {
            $item->setPurchaseUrl($data['purchaseUrl']);
        }
        
        if (isset($data['wishList'])) {
            try {
                $wishList = $this->wishListRepository->findOneById($data['wishList']);
                if (!$wishList) {
                    return new JsonResponse(
                        ['error' => 'WishList not found'],
                        Response::HTTP_BAD_REQUEST
                    );
                }
                $item->setWishList($wishList);
            } catch (\Exception $e) {
                return new JsonResponse(
                    ['error' => 'Invalid wishList: ' . $e->getMessage()],
                    Response::HTTP_BAD_REQUEST
                );
            }
        }

        // Validate using ValidatorInterface
        $errors = $this->validator->validate($item);
        
        if (count($errors) > 0) {
            $errorMessages = [];
            foreach ($errors as $error) {
                $errorMessages[$error->getPropertyPath()] = $error->getMessage();
            }
            return new JsonResponse(
                ['validation_errors' => $errorMessages],
                Response::HTTP_UNPROCESSABLE_ENTITY
            );
        }

        try {
            $this->entityManager->flush();

            return new JsonResponse([
                'message' => "sucessfully updated !",
                'item' => [
                    'id' => $item->getId(),
                    'title' => $item->getTitle(),
                    'price' => $item->getPrice(),
                    'wishListId' => $item->getWishList()->getId(),
                    'updatedAt' => (new \DateTimeImmutable())->format('Y-m-d H:i:s'),
                ],
                'path' => 'src/Controller/ItemController.php'
            ]);
        } catch (\Exception $e) {
            return new JsonResponse(
                ['error' => 'Failed to update item: ' . $e->getMessage()],
                Response::HTTP_INTERNAL_SERVER_ERROR
            );
        }
    }

    #[Route('/{id}', methods: ['DELETE'])]
    public function deleteItem(int $id): JsonResponse
    {
        $item = $this->itemRepository->findOneById($id);
        
        if (!$item) {
            return new JsonResponse(
                ['error' => 'Item not found'],
                Response::HTTP_NOT_FOUND
            );
        }

        try {
            $this->entityManager->remove($item);
            $this->entityManager->flush();

            return new JsonResponse(
                [
                    'message' => "successfully deleted",
                    'path' => 'src/Controller/ItemController.php',
                ],
                Response::HTTP_OK
            );
        } catch (\Exception $e) {
            return new JsonResponse(
                ['error' => 'Failed to delete item: ' . $e->getMessage()],
                Response::HTTP_INTERNAL_SERVER_ERROR
            );
        }
    }



}
